﻿using UnityEngine;
using System.IO;
using System;
using HarmonyLib;

public class SuperSoaker : Mod
{
    Harmony harmony;
    static string configPath = Path.Combine(SaveAndLoad.WorldPath, "SuperSoaker.json");
    public static JSONObject Config = getSaveJson();
    static public JsonModInfo modInfo;
    static public string returnPrefix
    {
        get
        {
            return "[" + modInfo.name + "]: ";
        }
    }
    public static float sprayMultiplier
    {
        get
        {
            if (Config.IsNull || !Config.HasField("sprayMulti"))
                return 2;
            return Config.GetField("sprayMulti").n;
        }
        set
        {
            if (!Config.IsNull && Config.HasField("sprayMulti"))
                Config.SetField("sprayMulti", value);
            else
                Config.AddField("sprayMulti", value);
            saveJson(Config);
        }
    }
    public void Start()
    {
        modInfo = modlistEntry.jsonmodinfo;
        harmony = new Harmony("com.aidanamite.SuperSoaker");
        harmony.PatchAll();
        Log("Mod has been loaded!");
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll();
        Log("Mod has been unloaded!");
    }

    public static void Log(object message)
    {
        Debug.Log(returnPrefix + message.ToString());
    }

    private static JSONObject getSaveJson()
    {
        JSONObject data;
        try
        {
            data = new JSONObject(File.ReadAllText(configPath));
        }
        catch
        {
            data = JSONObject.Create();
        }
        return data;
    }

    private static void saveJson(JSONObject data)
    {
        try
        {
            if (!data.IsNull)
                File.WriteAllText(configPath, data.ToString());
        }
        catch (Exception err)
        {
            Log("An error occured while trying to save settings: " + err.Message);
        }
    }

    [ConsoleCommand(name: "waterSprayMultiplier", docs: "Syntax: 'waterSprayMultiplier <multiplier>'  Changes the sprinkler's spray area")]
    public static string MyCommand(string[] args)
    {
        if (args.Length < 1)
            return "Not enough arguments";
        if (args.Length > 1)
            return "Too many arguments";
        try
        {
            sprayMultiplier = float.Parse(args[0]);
            return returnPrefix + "Multiplier id now " + sprayMultiplier;
        } catch
        {
            return returnPrefix + "Failed to parse " + args[0] + " as a number";
        }
    }

    public void ExtraSettingsAPI_SettingsOpen()
    {
        ExtraSettingsAPI_SetSliderValue("Spray Multiplier", sprayMultiplier);

    }

    public void ExtraSettingsAPI_SettingsClose()
    {
        sprayMultiplier = ExtraSettingsAPI_GetSliderRealValue("Spray Multiplier");
    }


    static Traverse ExtraSettingsAPI_Traverse;
    static bool ExtraSettingsAPI_Loaded = false;
    public float ExtraSettingsAPI_GetSliderRealValue(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            return ExtraSettingsAPI_Traverse.Method("getSliderRealValue", new object[] { this, SettingName }).GetValue<float>();
        return 0;
    }
    public void ExtraSettingsAPI_SetSliderValue(string SettingName, float value)
    {
        if (ExtraSettingsAPI_Loaded)
            ExtraSettingsAPI_Traverse.Method("setSliderValue", new object[] { this, SettingName, value }).GetValue<float>();
    }
}

[HarmonyPatch(typeof(Physics), "OverlapSphere", new Type[] { typeof(Vector3), typeof(float), typeof(int) })]
public class Patch_GetTargetCrops
{
    static void Prefix(ref float radius)
    {
        if (Environment.StackTrace.Contains("at Sprinkler"))
            radius *= SuperSoaker.sprayMultiplier;
    }
}